import { throwError } from './validation';

/**
 * 生成网站地图树形结构
 * 
 * @param {路由配置等} options 
 * @returns 
 */
function generateVueSitemapsTree(options) {
    // If a base URL is specified, make sure it ends with a slash
    const baseURL = options.baseURL ? `${options.baseURL.replace(/\/+$/, '')}/${options.hashMode ? '#/' : ''}` : '';

    const sitemapTree = generateTree(options.routes);
    console.log('sitemapTree1111', sitemapTree);
    fillUrl(sitemapTree, baseURL, options.trailingSlash);
    return sitemapTree;
}

/**
 * 生成完整url
 */
function fillUrl(sitemapTree, baseURL, trailingSlash) {

    sitemapTree.forEach(node => {
        node.url = escapeUrl(baseURL + node.url.replace(/^\//, '')).replace(/\/$/, '') + (trailingSlash ? '/' : '');
        if ('children' in node) {
            fillUrl(node.children, baseURL, trailingSlash);
        }
    })

}

function escapeUrl(url) {
    return encodeURI(url)
        .replace('&', '&amp;')
        .replace("'", '&apos;')
        .replace('"', '&quot;')
        .replace('<', '&lt;')
        .replace('>', '&gt;');
}

/**
 * 生成网站地图树形结构
 * 
 * @param {路由实例} routes 
 * @param {父路由} parentPath 
 * @param {*} parentMeta 
 * @returns 
 */
function generateTree(routes, parentPath = '', parentMeta = {}) {

    var result = [];
    routes.forEach(route => {

        var path = (route.path.startsWith('/') ? route.path : `${parentPath}/${route.path}`).replace(/^\/+/, '');
        const meta = { ...parentMeta, ...(route.meta ? (route.meta.sitemap || {}) : {}) };
        const params = (path.match(/:\w+(:?\(.+?\)|\?)?/g) || []).map(param => ({
            str: param,
            name: param.slice(1).replace(/\(.+?\)/, '').replace('?', ''),
            regexp: /\(.+?\)/.test(param) ? new RegExp(param.match(/\((.+?)\)/)[1]) : null,
            isOptional: param.endsWith('?'),
        }));

        /** Ignored route */
        if (meta.ignoreRoute || (route.path.includes('*') && !('loc' in meta))) return null;

        /** Static route */
        if ('loc' in meta) {
            path = meta.loc;
        }

        /** Dynamic route */
        if (params.length) {
            path = generateDynamicRoutePath(meta, params, path, route.path);
        }

        var node = generateTreeNode(route, path);
        if ('children' in route && route.children.length !== 0) {
            node.children = generateTree(route.children, path, meta);
        }
        result.push(node);
    });

    return result;

}

/**
 * 生成树节点
 * 
 * @param {路由} route 
 * @param {*} url 
 * @returns 
 */
function generateTreeNode(route, url) {
    return {
        url,
        disabled: 'children' in route && route.children.length !== 0,
        name: route.name ? route.name : '',
        title: route.meta ? (route.meta.title ? route.meta.title : '') : ''
    }
}

/**
 * 动态路由根据meta.slugs替换参数
 * 
 * @param {route.meta} meta 
 * @param {*} params 
 * @param {*} path 
 * @param {route.path} routePath 
 */
function generateDynamicRoutePath(meta, params, path, routePath) {

    if (!meta.slugs) throwError(`need slugs to generate URLs from dynamic route '${routePath}'`);

    let slugs = (typeof meta.slugs == 'function') ? meta.slugs.call() : meta.slugs;
    // validateSlugs(slugs, `invalid slug for route '${routePath}'`);

    slugs.map(async function (slug) {
        // Wrap the slug in an object if needed
        if (typeof slug != 'object') slug = { [params[0].name]: slug };

        // Replace each parameter by its corresponding value
        const loc = params.reduce(function (result, param) {
            // Check that the correct slug exists
            if (param.name in slug === false)
                throwError(`need slug for param '${param.name}' of route '${routePath}'`);

            // Check that the slug matched a potential regex pattern used to validate the param
            if (param.regexp && !param.regexp.test(slug[param.name].toString()))
                throwError(`the slug \`${slug[param.name]}\` for param '${param.name}' doesn't match its regex pattern ${param.regexp.toString()}`);

            return result.replace(param.str, slug[param.name]);
        }, path);

        return loc;
    });
}

export { generateVueSitemapsTree };
